/*
  Manual joystick controller for MPCNC
  V0.2, February 10, 2019
  (C) Armin Thinnes 2019
*/

// These constants won't change. They're used to give names to the pins used:
const int x_Pin = A1;  // Analog input pin that the x-potentiometer is attached to
const int y_Pin = A2;  // Analog input pin that the y-potentiometer is attached to
const int taste = 4; // joystick switch pin
const int blau = 5; // blue LED
const int gruen =6; // green LED
const int gelb = 7; // yello LED
const int rot = 8; // red LED
const int weiss = 9; // white LED
const int home_xy=10; // home_xy button pin
const int home_z=11; // home_z button pin
const int reset_coordinates=12; // reset_ccordinates button
const long baud_rate=250000;


boolean z_axis = false; //move z-axis if true, default is xy
boolean hxy = false; //define buttons_presed variables
boolean hz = false;
boolean rc = false;


const int button_Pin = 4; // Analog output pin that the LED is attached to

int x=0; //initialize variables
int y=0;
int z=0;
int t=0;
String X_command = "X-000.0 F200";
String Y_command = "Y-000.0 F200";
String Z_command = "Z-000.0 F200";
String vorschub = "100.10 ";
int toggle=5; // set 0.1mm as default, blue led (5)

void setup() {
  // initialize I/O pins
  
  pinMode(blau, OUTPUT);
  pinMode(gruen, OUTPUT);
  pinMode(gelb, OUTPUT);
  pinMode(rot, OUTPUT);
  pinMode(weiss, OUTPUT);
  pinMode(taste, INPUT_PULLUP);
  pinMode(home_xy, INPUT_PULLUP);
  pinMode(home_z, INPUT_PULLUP);
  pinMode(reset_coordinates, INPUT_PULLUP);
  
  digitalWrite(blau, HIGH);   // Lamp test
  digitalWrite(gruen,HIGH);   
  digitalWrite(gelb, HIGH);   
  digitalWrite(rot, HIGH);
  digitalWrite(weiss, HIGH);   
  delay(1000);                       // wait for a second
  digitalWrite(blau, LOW);   
  digitalWrite(gruen, LOW);   
  digitalWrite(gelb, LOW);   
  digitalWrite(rot, LOW);  
  digitalWrite(weiss, LOW); 
  
  delay(1000);;
                    
}

void loop() {
  // read the analog and digital in values:
  x = analogRead(x_Pin);
  y = analogRead(y_Pin);
  hxy = digitalRead(home_xy);
  hz = digitalRead(home_z);
  rc = digitalRead(reset_coordinates);
  t = digitalRead(taste);
  
  if (t==LOW)
  {
    toggle=toggle+1;
    if (toggle==9) // switch from Z-axis to xy
    {toggle=5;
    z_axis = !(z_axis);}
    
 }
  
  switch(toggle) {
    case 5:
      vorschub="0.10 "; //set movement to 0.1mm
      break;
    case 6: 
      vorschub="1.0 "; //set movement to 1mm
      break;  
    case 7: 
      vorschub="10.0 "; //set movement to 10mm
      break;  
    case 8: 
      if (z_axis) // maximum Z-axis movement is 20mm not 100. We don't want to shoot it out of the MPCNC!
      {
      vorschub="20.0 "; //set movement to 20mm
      }
      else
      {
        vorschub="100.0 "; //set movement to 100
        };
      break;  
    
  }
  x = map(x, 0,1020, 1, -1); //map analog readings to only -1, 0 and 1 (makes the analog joystick behave like a digital one)
  y = map(y, 0, 1020, -1, 1);
  z = y; // use y-potentiometer for z-axis operation - dualmode
    digitalWrite(weiss, z_axis); //light white if Z-axis is to be moved
    digitalWrite(blau, LOW);   
    digitalWrite(gruen, LOW);   
    digitalWrite(gelb, LOW);   
    digitalWrite(rot, LOW); 
    digitalWrite(toggle,HIGH); //set appropriate LED
  switch (x) {
    case -1: //left
      X_command = String(" X-" + vorschub);
      break;
    
    case 1://right
      X_command = String(" X" + vorschub);
      break;
      
    default:// middle position of x-potentiometer no move
      X_command =" X0 ";
      
    break;
  }
  
switch (y) {
    case -1: //forward
      Y_command = String(" Y-" + vorschub);
      break;
    case 1: //backward
      Y_command = String(" Y" + vorschub);
      break;
      
    default: // middle position of y-potentiometer no move
      Y_command =" Y0 ";
      
    break;
  }



switch (z) {
    case -1: //down
      Z_command = String(" Z-" + vorschub);
      break;
    case 1: // up
      Z_command = String(" Z" + vorschub);
      break;
      
    default: // middle position of z(y)-potentiometer no move
      Z_command =" Z0 ";
      // default is optional
    break;
  }
 if (Y_command==" Y0 " && X_command==" X0 ") 
 {
   }
 else
 {
  if (z_axis) 
   {
   switch (z) {
    case 0:
    break;
    default: //execute z-axis commands
    {
   Serial.begin(baud_rate); //open serial port
   Serial.println("G91");
  Serial.print("G1");
  Serial.print(Z_command);
  Serial.println("F1000");
  Serial.println("G90");
  Serial.println("M114");
  Serial.flush();
  Serial.end();//close serial port to avoid problems with Octoprint
   }; 
   }
   }
   else
  {
    Serial.begin(baud_rate); //open serial port
  Serial.println("G91");
  Serial.print("G1");
  Serial.print(X_command);
  Serial.print(Y_command);
  Serial.println("F1000");
  Serial.println("G90");
  Serial.println("M114");
  Serial.flush();
  Serial.end();//close serial port to avoid problems with Octoprint
  }
 }
 if (!(hxy)) 
 
 {
  Serial.begin(baud_rate); //open serial port
  Serial.println("G28 X Y");
  Serial.println("M117 Home XY...");
  Serial.end();//close serial port to avoid problems with Octoprint
  delay(1000);
 }; 
 
 if (!(hz))
 {Serial.begin(baud_rate);//open serial port
  Serial.println("G28 Z");
  Serial.println("M117 Home Z...");
  Serial.end();//close serial port to avoid problems with Octoprint
  delay(1000);
 };
 
 if (!(rc)) 
  {Serial.begin(baud_rate); //open serial port
    Serial.println("G92 X0 Y0 Z0");
   Serial.println("M300 S300 P1000");
   Serial.println("M117 Reset coordinates");
   Serial.end();//close serial port to avoid problems with Octoprint
   delay(1000);
  }
 
  // wait 350 milliseconds before the next loop to avoid too many moves cueing up
  delay(350);
}
